@@ -105,7 +105,8 @@ _gst_va_msg_send (OOP receiver,
   save = ap;
 #endif
 
-  for (numArgs = 0; (anArg = va_arg (ap, OOP)) != NULL; numArgs++);
+  for (numArgs = 0; va_arg (ap, OOP) != NULL; numArgs++)
+    ;
 
   if (numArgs != _gst_selector_num_args (selector))
     return (_gst_nil_oop);
@@ -207,7 +208,7 @@ _gst_va_msg_sendf (PTR resultPtr,
     return;
 
   args = (OOP *) alloca (sizeof (OOP) * numArgs);
-  for (i = -1, s = selectorBuf, fp = &fmt[2]; *fp; fp++)
+  for (i = -1, fp = &fmt[2]; *fp; fp++)
     {
       if (*fp != '%')
 	continue;
@@ -216,7 +217,7 @@ _gst_va_msg_sendf (PTR resultPtr,
       switch (*fp)
         {
         case 'i':
-          args[++i] = FROM_INT (va_arg (ap, long));
+          args[++i] = FROM_C_INT (va_arg (ap, long));
 	  INC_ADD_OOP (args[i]);
           break;
 
@@ -308,7 +309,7 @@ _gst_va_msg_sendf (PTR resultPtr,
       switch (fmt[1])
 	{
 	case 'i':
-	  *(int *) resultPtr = IS_NIL (result) ? 0 : TO_INT (result);
+	  *(int *) resultPtr = IS_NIL (result) ? 0 : TO_C_INT (result);
 	  break;
 
 	case 'c':
@@ -399,7 +400,7 @@ _gst_eval_code (const char *str)
     _gst_initialize (NULL, NULL, GST_NO_TTY);
 
   _gst_push_cstring (str);
-  _gst_parse_stream (false);
+  _gst_parse_stream (NULL);
   _gst_pop_stream (true);
 }
 
@@ -413,9 +414,10 @@ _gst_eval_expr (const char *str)
     _gst_initialize (NULL, NULL, GST_NO_TTY);
 
   _gst_push_cstring (str);
-  _gst_parse_stream (false);
+  _gst_parse_stream (NULL);
   _gst_pop_stream (true);
   result = _gst_last_returned_value;
+  INC_ADD_OOP (result);
 
   return (result);
 }
@@ -475,13 +477,22 @@ _gst_class_name_to_oop (const char *name)
 }
 
 
+OOP
+_gst_uint_to_oop (unsigned long int i)
+{
+  if (!_gst_smalltalk_initialized)
+    _gst_initialize (NULL, NULL, GST_NO_TTY);
+
+  return (FROM_C_ULONG (i));
+}
+
 OOP
 _gst_int_to_oop (long int i)
 {
   if (!_gst_smalltalk_initialized)
     _gst_initialize (NULL, NULL, GST_NO_TTY);
 
-  return (FROM_INT (i));
+  return (FROM_C_LONG (i));
 }
 
 OOP
@@ -626,8 +637,8 @@ _gst_oop_to_c (OOP oop)
   if (!_gst_smalltalk_initialized)
     _gst_initialize (NULL, NULL, GST_NO_TTY);
 
-  if (IS_INT (oop))
-    return (TO_INT (oop));
+  if (IS_C_LONG (oop) || IS_C_ULONG (oop))
+    return (TO_C_LONG (oop));
 
   else if (OOP_CLASS (oop) == _gst_true_class
 	   || OOP_CLASS (oop) == _gst_false_class)
@@ -653,7 +664,7 @@ _gst_oop_to_int (OOP oop)
   if (!_gst_smalltalk_initialized)
     _gst_initialize (NULL, NULL, GST_NO_TTY);
 
-  return (TO_INT (oop));
+  return (TO_C_LONG (oop));
 }
 
 long
