@@ -74,6 +74,7 @@ typedef struct cparam
     float floatVal;
     double doubleVal;
     long double longDoubleVal;
+    long long longLongVal;
     struct {
       PTR pPtrVal;
       PTR ptrVal;
@@ -195,6 +196,7 @@ static int my_lstat (const char *name,
 		     OOP out);
 #endif
 static int my_putenv (const char *str);
+static char **get_environ (void);
 static int my_chdir (const char *str);
 static int my_chown (const char *file, const char *owner, const char *group);
 static int my_symlink (const char* oldpath, const char* newpath);
@@ -252,6 +254,8 @@ static const char *c_type_name[] = {
   "wchar_t *",			/* CDATA_WSTRING */
   "wchar_t *",			/* CDATA_WSTRING_OUT */
   "char *",			/* CDATA_SYMBOL_OUT */
+  "long long",			/* CDATA_LONGLONG */
+  "unsigned long long", 	/* CDATA_ULONGLONG */
 };
 
 /* The errno on output from a callout */
@@ -392,6 +396,20 @@ my_putenv (const char *str)
   return (putenv (clone));
 }
 
+/*
+ * On FreeBSD and other BSDs there is the environ but it is not
+ * declared in header. Import it like this.
+ */
+#if !HAVE_DECL_ENVIRON
+extern char **environ;
+#endif
+
+static char **
+get_environ (void)
+{
+  return environ;
+}
+
 
 int
 my_chdir (const char *dir)
@@ -432,6 +450,13 @@ my_opendir (const char *dir)
   return (result);
 }
 
+long long
+test_longlong (long long aVerylongInt)
+{
+  printf ("Getting a long long 0x%llx\n", aVerylongInt);
+  return aVerylongInt;
+}
+
 void
 test_callin (OOP oop, int(*callback)(const char *))
 {
@@ -491,6 +516,17 @@ dld_open (const char *filename)
   handle = lt_dlopen (filename);
   if (!handle)
     handle = lt_dlopenext (filename);
+#ifdef __APPLE__
+  if (!handle)
+    {
+      /* For some reason, lt_dlopenext on OS X doesn't try ".dylib" as
+         a possible extension, so we're left with trying it here. */
+      char *full_filename;
+      asprintf(&full_filename, "%s.dylib", filename);
+      handle = lt_dlopen (full_filename);
+      free (full_filename);
+    }
+#endif
   if (handle)
     {
       initModule = lt_dlsym (handle, "gst_initModule");
@@ -585,6 +621,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("getArgv", get_argv);
 
   /* Test functions */
+  _gst_define_cfunc ("testLongLong", test_longlong);
   _gst_define_cfunc ("testCallin", test_callin);
   _gst_define_cfunc ("testCString", test_cstring);
   _gst_define_cfunc ("testCObjectPtr", test_cobject_ptr);
@@ -592,6 +629,7 @@ _gst_init_cfuncs (void)
   /* Access to C library */
   _gst_define_cfunc ("system", system);
   _gst_define_cfunc ("getenv", getenv);
+  _gst_define_cfunc ("environ", get_environ);
   _gst_define_cfunc ("putenv", my_putenv);
   _gst_define_cfunc ("printf", printf);
 
@@ -611,6 +649,7 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("rewinddir", rewinddir);
   _gst_define_cfunc ("extractDirentName", extract_dirent_name);
 
+  _gst_define_cfunc ("link", link);
   _gst_define_cfunc ("symlink", my_symlink);
   _gst_define_cfunc ("unlink", unlink);
   _gst_define_cfunc ("rename", rename);
@@ -619,6 +658,13 @@ _gst_init_cfuncs (void)
   _gst_define_cfunc ("mkdir", my_mkdir);
   _gst_define_cfunc ("mkdtemp", my_mkdtemp);
   _gst_define_cfunc ("getCurDirName", _gst_get_cur_dir_name);
+  _gst_define_cfunc ("fsync", fsync);
+#if HAVE_DECL_FDATASYNC
+  _gst_define_cfunc ("fdatasync", fdatasync);
+#else
+  _gst_define_cfunc ("fdatasync", fsync);
+#endif
+  _gst_define_cfunc ("sync", sync);
 
   _gst_define_cfunc ("fileIsReadable", _gst_file_is_readable);
   _gst_define_cfunc ("fileIsWriteable", _gst_file_is_writeable);
@@ -719,6 +765,11 @@ _gst_c_type_size (int type)
     case CDATA_ULONG:
       return sizeof (unsigned long);
 
+    case CDATA_LONGLONG:
+      return sizeof (long long);
+    case CDATA_ULONGLONG:
+      return sizeof (unsigned long long);
+
     case CDATA_FLOAT:
       return sizeof (float);
     case CDATA_DOUBLE:
@@ -823,6 +874,11 @@ _gst_invoke_croutine (OOP cFuncOOP,
         case CDATA_WSTRING_OUT:
         case CDATA_STRING_OUT:
         case CDATA_BYTEARRAY_OUT:
+        case CDATA_SYMBOL_OUT:
+        case CDATA_STRING:
+        case CDATA_BYTEARRAY:
+        case CDATA_SYMBOL:
+        case CDATA_WSTRING:
 	  needPostprocessing = true;
 	  /* fall through */
 
@@ -968,6 +1024,9 @@ get_ffi_type (OOP returnTypeOOP)
 #else
       return &ffi_type_sint64;
 #endif
+    case CDATA_LONGLONG:
+    case CDATA_ULONGLONG:
+      return &ffi_type_sint64;
 
     case CDATA_VOID:
     case CDATA_INT:
@@ -1036,9 +1095,13 @@ smalltalk_to_c (OOP oop,
     {
       switch (cType)
         {
+       case CDATA_LONGLONG:
+       case CDATA_ULONGLONG:
+         cp->u.longLongVal = to_c_int_64 (oop);
+         return &ffi_type_sint64;
         case CDATA_LONG:
-	case CDATA_ULONG:
-	  cp->u.longVal = TO_C_LONG (oop);
+       case CDATA_ULONG:
+         cp->u.longVal = TO_C_LONG (oop);
 #if LONG_MAX == 2147483647
           return &ffi_type_sint32;
 #else
@@ -1083,9 +1146,13 @@ smalltalk_to_c (OOP oop,
     {
       switch (cType)
         {
+       case CDATA_LONGLONG:
+       case CDATA_ULONGLONG:
+         cp->u.longLongVal = (long long)(oop == _gst_true_oop);
+         return &ffi_type_sint64;
         case CDATA_LONG:
-	case CDATA_ULONG:
-	  cp->u.longVal = (oop == _gst_true_oop);
+       case CDATA_ULONG:
+         cp->u.longVal = (oop == _gst_true_oop);
 #if LONG_MAX == 2147483647
           return &ffi_type_sint32;
 #else
@@ -1298,6 +1365,14 @@ c_to_smalltalk (cparam *result, OOP receiverOOP, OOP returnTypeOOP)
       resultOOP = FROM_C_ULONG (result->u.longVal);
       break;
 
+    case CDATA_LONGLONG:
+      resultOOP = FROM_C_LONGLONG (result->u.longLongVal);
+      break;
+
+    case CDATA_ULONGLONG:
+      resultOOP = FROM_C_ULONGLONG (result->u.longLongVal);
+      break;
+
     case CDATA_STRING:
     case CDATA_STRING_OUT:
     case CDATA_WSTRING:
