@@ -162,6 +162,7 @@ static const char standard_files[] = {
   "SysDict.st\0"
   "Object.st\0"
   "Message.st\0"
+  "MessageLookup.st\0"
   "DirMessage.st\0"
   "Boolean.st\0"
   "False.st\0"
@@ -176,13 +177,13 @@ static const char standard_files[] = {
   "Date.st\0"
   "Time.st\0"
   "Number.st\0"
+  "SmallInt.st\0"
   "Float.st\0"
   "FloatD.st\0"
   "FloatE.st\0"
   "FloatQ.st\0"
   "Fraction.st\0"
   "LargeInt.st\0"
-  "SmallInt.st\0"
   "Character.st\0"
   "UniChar.st\0"
   "Link.st\0"
@@ -199,6 +200,10 @@ static const char standard_files[] = {
   "CompiledBlk.st\0"
   "Array.st\0"
   "ByteArray.st\0"
+  "ContextPart.st\0"
+  "MthContext.st\0"
+  "BlkContext.st\0"
+  "BlkClosure.st\0"
   "CharArray.st\0"
   "String.st\0"
   "Symbol.st\0"
@@ -224,10 +229,6 @@ static const char standard_files[] = {
   "RWStream.st\0"
   "UndefObject.st\0"
   "ProcSched.st\0"
-  "ContextPart.st\0"
-  "MthContext.st\0"
-  "BlkContext.st\0"
-  "BlkClosure.st\0"
   "Behavior.st\0"
   "ClassDesc.st\0"
   "Class.st\0"
@@ -235,10 +236,10 @@ static const char standard_files[] = {
   "Continuation.st\0"
   "Memory.st\0"
   "MethodInfo.st\0"
+  "DebugInformation.st\0"
   "FileSegment.st\0"
   "FileDescr.st\0"
   "SymLink.st\0"
-  "Security.st\0"
   "WeakObjects.st\0"
   "ObjMemory.st\0"
 
@@ -281,14 +282,15 @@ static const char standard_files[] = {
   "FileStream.st\0"
 
   /* Goodies */
-  "Autoload.st\0"
+  "DynVariable.st\0"
   "DLD.st\0"
   "Getopt.st\0"
   "Generator.st\0"
   "StreamOps.st\0"
-  "ObjDumper.st\0"
-  "PkgLoader.st\0"
   "Regex.st\0"
+  "PkgLoader.st\0"
+  "DirPackage.st\0"
+  "Autoload.st\0"
 };
 
 /* The argc and argv that are passed to libgst via gst_smalltalk_args. 
@@ -455,6 +457,7 @@ _gst_initialize (const char *kernel_dir,
 
   _gst_init_sysdep ();
   _gst_init_signals ();
+  _gst_init_event_loop();
   _gst_init_cfuncs ();
   _gst_init_sockets ();
   _gst_init_primitives ();
@@ -495,7 +498,6 @@ _gst_initialize (const char *kernel_dir,
   if (loadBinary && _gst_load_from_file (_gst_binary_image_name))
     {
       _gst_init_interpreter ();
-      _gst_init_compiler ();
       _gst_init_vmproxy ();
     }
   else if (abortOnFailure)
@@ -513,7 +515,6 @@ _gst_initialize (const char *kernel_dir,
       _gst_init_mem_default ();
       _gst_init_dictionary ();
       _gst_init_interpreter ();
-      _gst_init_compiler ();
       _gst_init_vmproxy ();
 
       _gst_install_initial_methods ();
