@@ -1,20 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.4.3.  */
+/* A Bison parser, made by GNU Bison 3.0.2.  */
+
+/* Bison interface for Yacc-like parsers in C
+
+   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
 
-/* Skeleton interface for Bison's Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-   2009, 2010 Free Software Foundation, Inc.
-   
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -27,26 +26,34 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+#ifndef YY_YY_GENVM_PARSE_H_INCLUDED
+# define YY_YY_GENVM_PARSE_H_INCLUDED
+/* Debug traces.  */
+#ifndef YYDEBUG
+# define YYDEBUG 1
+#endif
+#if YYDEBUG
+extern int yydebug;
+#endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     ID = 258,
-     EXPR = 259,
-     NUMBER = 260,
-     VM_OPERATION = 261,
-     VM_TABLE = 262,
-     VM_BYTECODE = 263,
-     VM_DOTS = 264,
-     VM_MINUSMINUS = 265
-   };
+  enum yytokentype
+  {
+    ID = 258,
+    EXPR = 259,
+    NUMBER = 260,
+    VM_OPERATION = 261,
+    VM_TABLE = 262,
+    VM_BYTECODE = 263,
+    VM_DOTS = 264,
+    VM_MINUSMINUS = 265
+  };
 #endif
 /* Tokens.  */
 #define ID 258
@@ -58,15 +65,12 @@
 #define VM_DOTS 264
 #define VM_MINUSMINUS 265
 
-
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+typedef union YYSTYPE YYSTYPE;
+union YYSTYPE
 {
-
-/* Line 1685 of yacc.c  */
-#line 127 "genvm-parse.y"
+#line 127 "genvm-parse.y" /* yacc.c:1909  */
 
   struct operation_list *oplist;
   struct operation_info *op;
@@ -76,16 +80,15 @@ typedef union YYSTYPE
   char *text;
   int num;
 
-
-
-/* Line 1685 of yacc.c  */
-#line 83 "../../libgst/genvm-parse.h"
-} YYSTYPE;
+#line 84 "genvm-parse.h" /* yacc.c:1909  */
+};
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
+
 extern YYSTYPE yylval;
 
+int yyparse (void);
 
+#endif /* !YY_YY_GENVM_PARSE_H_INCLUDED  */
