@@ -106,6 +106,7 @@ typedef struct input_stream
   const char *prompt;
 
   OOP fileOOP;			/* the object stored in FileSegments */
+  mst_Boolean freeFileName;
   const char *fileName;
   off_t fileOffset;
 
@@ -240,6 +241,8 @@ _gst_pop_stream (mst_Boolean closeIt)
       break;
     }
 
+  if (stream->freeFileName)
+   xfree ((char *) stream->fileName);
   xfree (stream);
 }
 
@@ -335,6 +338,7 @@ push_new_stream (stream_type type)
   newStream->column = 0;
   newStream->fileOffset = -1;
   newStream->type = type;
+  newStream->freeFileName = false;
   newStream->fileName = NULL;
   newStream->prompt = NULL;
   newStream->fileOOP = _gst_nil_oop;
@@ -358,7 +362,10 @@ _gst_set_stream_info (int line,
   in_stream->fileOffset = fileOffset;
 
   if (!IS_NIL (fileNameOOP))
-    in_stream->fileName = _gst_to_cstring (fileNameOOP);
+    {
+      in_stream->fileName = _gst_to_cstring (fileNameOOP);
+      in_stream->freeFileName = true;
+    }
 }
  
 void
@@ -407,7 +414,10 @@ my_getc (input_stream stream)
 	  char *buf;
 	  _gst_msg_sendf(&buf, "%s %o nextAvailable: %i", stream->st_oop.oop, 1024);
 	  if (!buf || !*buf)
-	    return EOF;
+            {
+              xfree (buf);
+	      return EOF;
+            }
 
 	  refill_stream (stream, buf, false);
 	}
@@ -558,7 +568,7 @@ _gst_get_source_string (off_t startPos, off_t endPos)
 OOP
 get_cur_file (void)
 {
-  const char *fullFileName;
+  char *fullFileName;
 
   if (!in_stream)
     return _gst_nil_oop;
@@ -576,6 +586,7 @@ get_cur_file (void)
       _gst_get_full_file_name (in_stream->fileName);
 
   in_stream->fileOOP = _gst_string_new (fullFileName);
+  xfree (fullFileName);
   _gst_register_oop (in_stream->fileOOP);
   return (in_stream->fileOOP);
 }
@@ -815,7 +826,7 @@ _gst_process_stdin (const char *prompt)
   _gst_push_stdin_string ();
   if (isatty (0))
     in_stream->prompt = prompt;
-  _gst_parse_stream (false);
+  _gst_parse_stream (NULL);
   _gst_pop_stream (true);
   _gst_non_interactive = true;
 }
@@ -823,7 +834,6 @@ _gst_process_stdin (const char *prompt)
 mst_Boolean
 _gst_process_file (const char *fileName, enum gst_file_dir dir)
 {
-  enum undeclared_strategy old;
   int fd;
   char *f;
 
@@ -841,11 +851,9 @@ _gst_process_file (const char *fileName, enum gst_file_dir dir)
       if (_gst_verbosity == 3)
 	printf ("Processing %s\n", f);
 
-      old = _gst_set_undeclared (UNDECLARED_GLOBALS);
       _gst_push_unix_file (fd, f);
-      _gst_parse_stream (false);
+      _gst_parse_stream (NULL);
       _gst_pop_stream (true);
-      _gst_set_undeclared (old);
       errno = 0;
     }
 
@@ -1142,13 +1150,13 @@ _gst_initialize_readline (void)
   rl_special_prefixes = (char *) "+-=*<>~?%/@|&\\";
 
   /* Our rules for quoting are a bit different from the default */
-  rl_filename_quoting_function = (CPFunction *) readline_quote_filename;
+  rl_filename_quoting_function = (rl_quote_func_t *) readline_quote_filename;
   rl_filename_dequoting_function =
-    (CPFunction *) readline_dequote_filename;
+    (rl_dequote_func_t *) readline_dequote_filename;
 
   /* Try to match a symbol before a filename */
   rl_attempted_completion_function =
-    (CPPFunction *) readline_match_symbols;
+    (rl_completion_func_t *) readline_match_symbols;
 
   /* Since we have to sort the array to perform the binary search,
      remove duplicates and avoid that readline resorts the result.  */
