@@ -113,13 +113,6 @@
    to speed up these messages for Arrays, Strings, and ByteArrays.  */
 #define METHOD_CACHE_SIZE		(1 << 11)
 
-typedef struct async_queue_entry
-{
-  void (*func) (OOP);
-  OOP data;
-}
-async_queue_entry;
-
 typedef struct interp_jmp_buf
 {
   jmp_buf jmpBuf;
@@ -247,14 +240,10 @@ static int class_cache_prim;
 #endif
 
 /* Queue for async (outside the interpreter) semaphore signals */
-gl_lock_define (static, async_queue_lock);
 static mst_Boolean async_queue_enabled = true;
-static int async_queue_index;
-static int async_queue_size;
-static async_queue_entry *queued_async_signals;
-
-static int async_queue_index_sig;
-static async_queue_entry queued_async_signals_sig[NSIG];
+static async_queue_entry queued_async_signals_tail;
+static async_queue_entry *queued_async_signals = &queued_async_signals_tail;
+static async_queue_entry *queued_async_signals_sig = &queued_async_signals_tail;
 
 /* When not NULL, this causes the byte code interpreter to immediately
    send the message whose selector is here to the current stack
@@ -295,6 +284,15 @@ static inline mst_Boolean cached_index_oop_put_primitive (OOP rec,
 							  OOP val,
 							  intptr_t spec);
 
+/* This functions accepts an OOP for a Semaphore object and puts the
+   PROCESSOOP to sleep, unless the semaphore has excess signals
+   on it.  */
+static void sync_wait_process (OOP semaphoreOOP,
+			       OOP processOOP);
+
+/* Empty the queue of asynchronous calls.  */
+static void empty_async_queue (void);
+
 /* Try to find another process with higher or same priority as the
    active one.  Return whether there is one.  */
 static mst_Boolean would_reschedule_process (void);
@@ -333,7 +331,13 @@ static void set_preemption_timer (void);
 
 /* Same as _gst_parse_stream, but creating a reentrancy_jmpbuf.  Returns
    true if interrupted. */
-static mst_Boolean parse_stream_with_protection (mst_Boolean method);
+static mst_Boolean parse_stream_with_protection (OOP currentNamespace);
+
+/* Same as _gst_parse_method_from_stream, but creating a reentrancy_jmpbuf.
+   Returns true if interrupted, pushes the last compiled method on the
+   stack. */
+static mst_Boolean parse_method_from_stream_with_protection (OOP currentClass,
+							     OOP currentCategory);
 
 /* Put the given process to sleep by rotating the list of processes for
    PROCESSOOP's priority (i.e. it was the head of the list and becomes
@@ -578,21 +582,6 @@ static void * const *dispatch_vec;
 #define PARENT_CONTEXT(contextOOP) \
   ( ((gst_method_context) OOP_TO_OBJ (contextOOP)) ->parentContext)
 
-/* Set whether the old context was a trusted one.  Untrusted contexts
-   are those whose receiver or sender is untrusted.  */
-#define UPDATE_CONTEXT_TRUSTFULNESS(contextOOP, parentContextOOP) \
-  MAKE_OOP_UNTRUSTED (contextOOP, \
-    IS_OOP_UNTRUSTED (_gst_self) | \
-    IS_OOP_UNTRUSTED (parentContextOOP));
-
-/* Set whether the current context is an untrusted one.  Untrusted contexts
-   are those whose receiver or sender is untrusted.  */
-#define IS_THIS_CONTEXT_UNTRUSTED() \
-  (UPDATE_CONTEXT_TRUSTFULNESS(_gst_this_context_oop, \
-			       PARENT_CONTEXT (_gst_this_context_oop)) \
-     & F_UNTRUSTED)
-
-
 /* Context management
  
    The contexts make up a linked list.  Their structure is:
@@ -765,8 +754,6 @@ empty_context_stack (void)
   context->spOffset = FROM_INT (sp - context->contextStack);
   context->ipOffset = FROM_INT (ip - method_base);
 
-  UPDATE_CONTEXT_TRUSTFULNESS (_gst_this_context_oop, context->parentContext);
-
   /* Even if the JIT is active, the current context might have no
      attached native_ip -- in fact it has one only if we are being
      called from activate_new_context -- so we have to `invent'
@@ -859,7 +846,6 @@ activate_new_context (int size,
     FROM_INT ((sp - thisContext->contextStack) - sendArgs);
   thisContext->ipOffset = FROM_INT (ip - method_base);
 
-  UPDATE_CONTEXT_TRUSTFULNESS (_gst_this_context_oop, thisContext->parentContext);
   _gst_this_context_oop = oop;
 
   return (newContext);
@@ -934,6 +920,8 @@ lookup_method (OOP sendSelector,
 {
   inc_ptr inc;
   OOP argsArrayOOP;
+  gst_message_lookup messageLookup;
+  OOP messageLookupOOP;
 
   if (_gst_find_method (method_class, sendSelector, methodData))
     return (true);
@@ -941,8 +929,16 @@ lookup_method (OOP sendSelector,
   inc = INC_SAVE_POINTER ();
   argsArrayOOP = create_args_array (sendArgs);
   INC_ADD_OOP (argsArrayOOP);
-  PUSH_OOP (_gst_message_new_args (sendSelector, argsArrayOOP));
+
+  messageLookup = (gst_message_lookup) new_instance (_gst_message_lookup_class,
+					             &messageLookupOOP);
+
+  messageLookup->selector = sendSelector;
+  messageLookup->args = argsArrayOOP;
+  messageLookup->startingClass = method_class;
+  PUSH_OOP (messageLookupOOP);
   INC_RESTORE_POINTER (inc);
+
   return (false);
 }
 
@@ -1243,6 +1239,12 @@ disable_non_unwind_contexts (OOP returnContextOOP)
 
       /* Descend in the chain...  */
       newContextOOP = oldContext->parentContext;
+      if (IS_NIL(newContextOOP))
+        {
+            *chain = _gst_nil_oop;
+            return false;
+        }
+
       newContext = (gst_method_context) OOP_TO_OBJ (newContextOOP);
 
       /* This context cannot be deallocated in a LIFO way.  We must
@@ -1383,51 +1385,6 @@ get_scheduled_process (void)
   return (processor->activeProcess);
 }
 
-void
-add_first_link (OOP semaphoreOOP,
-		OOP processOOP)
-{
-  gst_semaphore sem;
-  gst_process process, lastProcess;
-  OOP lastProcessOOP;
-
-  process = (gst_process) OOP_TO_OBJ (processOOP);
-  if (!IS_NIL (process->myList))
-    {
-      sem = (gst_semaphore) OOP_TO_OBJ (process->myList);
-      if (sem->firstLink == processOOP)
-	{
-	  sem->firstLink = process->nextLink;
-	  if (sem->lastLink == processOOP)
-	    {
-	      /* It was the only process in the list */
-	      sem->lastLink = _gst_nil_oop;
-	    }
-	}
-      else if (sem->lastLink == processOOP)
-	{
-	  /* Find the new last link */
-	  lastProcessOOP = sem->firstLink;
-	  lastProcess = (gst_process) OOP_TO_OBJ (lastProcessOOP);
-	  while (lastProcess->nextLink != processOOP)
-	    {
-	      lastProcessOOP = lastProcess->nextLink;
-	      lastProcess = (gst_process) OOP_TO_OBJ (lastProcessOOP);
-	    }
-	  sem->lastLink = lastProcessOOP;
-	  lastProcess->nextLink = _gst_nil_oop;
-	}
-    }
-
-  sem = (gst_semaphore) OOP_TO_OBJ (semaphoreOOP);
-  process->myList = semaphoreOOP;
-  process->nextLink = sem->firstLink;
-
-  sem->firstLink = processOOP;
-  if (IS_NIL (sem->lastLink))
-    sem->lastLink = processOOP;
-}
-
 static void
 remove_process_from_list (OOP processOOP)
 {
@@ -1452,7 +1409,7 @@ remove_process_from_list (OOP processOOP)
         }
       else
         {
-          /* Find the new last link */
+          /* Find the new prev node */
           lastProcessOOP = sem->firstLink;
           lastProcess = (gst_process) OOP_TO_OBJ (lastProcessOOP);
           while (lastProcess->nextLink != processOOP)
@@ -1472,6 +1429,25 @@ remove_process_from_list (OOP processOOP)
   process->nextLink = _gst_nil_oop;
 }
 
+void
+add_first_link (OOP semaphoreOOP,
+		OOP processOOP)
+{
+  gst_semaphore sem;
+  gst_process process;
+
+  process = (gst_process) OOP_TO_OBJ (processOOP);
+  remove_process_from_list (processOOP);
+
+  sem = (gst_semaphore) OOP_TO_OBJ (semaphoreOOP);
+  process->myList = semaphoreOOP;
+  process->nextLink = sem->firstLink;
+
+  sem->firstLink = processOOP;
+  if (IS_NIL (sem->lastLink))
+    sem->lastLink = processOOP;
+}
+
 void
 suspend_process (OOP processOOP)
 {
@@ -1499,32 +1475,7 @@ add_last_link (OOP semaphoreOOP,
   OOP lastProcessOOP;
 
   process = (gst_process) OOP_TO_OBJ (processOOP);
-  if (!IS_NIL (process->myList))
-    {
-      sem = (gst_semaphore) OOP_TO_OBJ (process->myList);
-      if (sem->firstLink == processOOP)
-	{
-	  sem->firstLink = process->nextLink;
-	  if (sem->lastLink == processOOP)
-	    {
-	      /* It was the only process in the list */
-	      sem->lastLink = _gst_nil_oop;
-	    }
-	}
-      else if (sem->lastLink == processOOP)
-	{
-	  /* Find the new last link */
-	  lastProcessOOP = sem->firstLink;
-	  lastProcess = (gst_process) OOP_TO_OBJ (lastProcessOOP);
-	  while (lastProcess->nextLink != processOOP)
-	    {
-	      lastProcessOOP = lastProcess->nextLink;
-	      lastProcess = (gst_process) OOP_TO_OBJ (lastProcessOOP);
-	    }
-	  sem->lastLink = lastProcessOOP;
-	  lastProcess->nextLink = _gst_nil_oop;
-	}
-    }
+  remove_process_from_list (processOOP);
 
   sem = (gst_semaphore) OOP_TO_OBJ (semaphoreOOP);
   process->myList = semaphoreOOP;
@@ -1602,81 +1553,147 @@ _gst_sync_signal (OOP semaphoreOOP, mst_Boolean incr_if_empty)
   return true;
 }
 
-static void
-do_async_signal (OOP semaphoreOOP)
+void
+_gst_do_async_signal (OOP semaphoreOOP)
 {
   _gst_sync_signal (semaphoreOOP, true);
 }
 
+void
+_gst_do_async_signal_and_unregister (OOP semaphoreOOP)
+{
+  _gst_sync_signal (semaphoreOOP, true);
+  _gst_unregister_oop (semaphoreOOP);
+}
+
+/* Async-signal-safe version, does no allocation.  Using an atomic operation
+   is still the simplest choice, but on top of that we check that the entry
+   is not already in the list.  Also, the datum and next field are NULLed
+   automatically when the call is made.  */
+void
+_gst_async_call_internal (async_queue_entry *e)
+{
+  /* For async-signal safety, we need to check that the entry is not
+     already in the list.  Checking that atomically with CAS is the
+     simplest way.  */
+  do
+    if (__sync_val_compare_and_swap(&e->next, NULL, queued_async_signals_sig))
+      return;
+  while (!__sync_bool_compare_and_swap (&queued_async_signals_sig, e->next, e));
+  SET_EXCEPT_FLAG (true);
+}
+
 void
 _gst_async_call (void (*func) (OOP), OOP arg)
 {
-  if (_gst_signal_count)
-    {
-      /* Async-signal-safe version.  Because the discriminant is
-	 _gst_signal_count, can only be called from within libgst.la
-	 (in particular events.c).  Also, because of that we assume
-	 interrupts are already disabled.  */
+  /* Thread-safe version for the masses.  This lockless stack
+     is reversed in the interpreter loop to get FIFO behavior.  */
+  async_queue_entry *sig = xmalloc (sizeof (async_queue_entry));
+  sig->func = func;
+  sig->data = arg;
 
-      if (async_queue_index_sig == NSIG)
-        {
-	  static const char errmsg[] = "Asynchronous signal queue full!\n";
-	  write (2, errmsg, sizeof (errmsg) - 1);
-	  abort ();
-	}
-
-      queued_async_signals_sig[async_queue_index_sig].func = func;
-      queued_async_signals_sig[async_queue_index_sig++].data = arg;
-    }
-  else
-    {
-      /* Thread-safe version for the masses.  */
-
-      gl_lock_lock (async_queue_lock);
-      if (async_queue_index == async_queue_size)
-        {
-          async_queue_size *= 2;
-          queued_async_signals =
-	    realloc (queued_async_signals,
-		 sizeof (async_queue_entry) * async_queue_size);
-	}
-
-      queued_async_signals[async_queue_index].func = func;
-      queued_async_signals[async_queue_index++].data = arg;
-      gl_lock_unlock (async_queue_lock);
-      _gst_wakeup ();
-    }
-  
+  do
+    sig->next = queued_async_signals;
+  while (!__sync_bool_compare_and_swap (&queued_async_signals,
+                                        sig->next, sig));
+  _gst_wakeup ();
   SET_EXCEPT_FLAG (true);
 }
 
 mst_Boolean
 _gst_have_pending_async_calls ()
 {
-  return async_queue_index_sig > 0 || async_queue_index > 0;
+  return (queued_async_signals != &queued_async_signals_tail
+          || queued_async_signals_sig != &queued_async_signals_tail);
+}
+
+void
+empty_async_queue ()
+{
+  async_queue_entry *sig, *sig_reversed;
+
+  /* Process a batch of asynchronous requests.  These are pushed
+     in LIFO order by _gst_async_call.  By reversing the list
+     in place before walking it, we get FIFO order.  */
+  sig = __sync_swap (&queued_async_signals, &queued_async_signals_tail);
+  sig_reversed = &queued_async_signals_tail;
+  while (sig != &queued_async_signals_tail)
+    {
+      async_queue_entry *next = sig->next;
+      sig->next = sig_reversed;
+      sig_reversed = sig;
+      sig = next;
+    }
+
+  sig = sig_reversed;
+  while (sig != &queued_async_signals_tail)
+    {
+      async_queue_entry *next = sig->next;
+      sig->func (sig->data);
+      free (sig);
+      sig = next;
+    }
+
+  /* For async-signal-safe processing, we need to avoid entering
+     the same item twice into the list.  So we use NEXT to mark
+     items that have been added...  */
+  sig = __sync_swap (&queued_async_signals_sig, &queued_async_signals_tail);
+  sig_reversed = &queued_async_signals_tail;
+  while (sig != &queued_async_signals_tail)
+    {
+      async_queue_entry *next = sig->next;
+      sig->next = sig_reversed;
+      sig_reversed = sig;
+      sig = next;
+    }
+
+  sig = sig_reversed;
+  while (sig != &queued_async_signals_tail)
+    {
+      async_queue_entry *next = sig->next;
+      void (*func) (OOP) = sig->func;
+      OOP data = sig->data;
+      barrier ();
+
+      sig->data = NULL;
+      barrier ();
+
+      /* ... and we only NULL it after a signal handler can start
+         writing to it.  */
+      sig->next = NULL;
+      barrier ();
+      func (data);
+      sig = next;
+    }
 }
 
 void
 _gst_async_signal (OOP semaphoreOOP)
 {
-  _gst_async_call (do_async_signal, semaphoreOOP);
+  _gst_async_call (_gst_do_async_signal, semaphoreOOP);
 }
 
 void
 _gst_async_signal_and_unregister (OOP semaphoreOOP)
 {
-  _gst_disable_interrupts (false);	/* block out everything! */
-  _gst_async_call (do_async_signal, semaphoreOOP);
-  _gst_async_call (_gst_unregister_oop, semaphoreOOP);
-  _gst_enable_interrupts (false);
+  _gst_async_call (_gst_do_async_signal_and_unregister, semaphoreOOP);
 }
 
 void
-_gst_sync_wait (OOP semaphoreOOP)
+sync_wait_process (OOP semaphoreOOP, OOP processOOP)
 {
   gst_semaphore sem;
+  mst_Boolean isActive;
 
   sem = (gst_semaphore) OOP_TO_OBJ (semaphoreOOP);
+  if (IS_NIL (processOOP))
+    {
+      processOOP = get_active_process ();
+      isActive = true;
+    }
+  else
+    isActive = (processOOP == get_active_process ());
+
   if (TO_INT (sem->signals) <= 0)
     {
       /* Have to suspend.  Prepare return value for #wait and move
@@ -1685,8 +1702,9 @@ _gst_sync_wait (OOP semaphoreOOP)
          Tweaking the stack top means that this function should only
 	 be called from a primitive.  */
       SET_STACKTOP (_gst_nil_oop);
-      add_last_link (semaphoreOOP, get_active_process ());
-      if (IS_NIL (ACTIVE_PROCESS_YIELD ()))
+      remove_process_from_list (processOOP);
+      add_last_link (semaphoreOOP, processOOP);
+      if (isActive && IS_NIL (ACTIVE_PROCESS_YIELD ()))
         {
 	  printf ("No runnable process");
 	  activate_process (_gst_prepare_execution_environment ());
@@ -1698,6 +1716,12 @@ _gst_sync_wait (OOP semaphoreOOP)
   /* printf ("wait %O %O\n", semaphoreOOP, sem->firstLink); */
 }
 
+void
+_gst_sync_wait (OOP semaphoreOOP)
+{
+  sync_wait_process (semaphoreOOP, _gst_nil_oop);
+}
+
 OOP
 remove_first_link (OOP semaphoreOOP)
 {
@@ -1770,7 +1794,8 @@ resume_process (OOP processOOP,
     {
       /* We're resuming a process with a *equal or higher* priority, so sleep
          the current one and activate the new one */
-      sleep_process (activeOOP);
+      if (active->myList != _gst_nil_oop)
+        sleep_process (activeOOP);
       activate_process (processOOP);
     }
   else
@@ -1983,8 +2008,12 @@ _gst_check_process_state (void)
 	  if (process->myList != processListOOP)
 	    abort ();
 
+#if 0
+	  /* This is false when a process has just finished initializing
+	     itself.  */
 	  if (process->priority != FROM_INT (priority))
 	    abort ();
+#endif
 
           /* Sanity check the last link in the process list.  */
 	  if (IS_NIL (process->nextLink) && processOOP != processList->lastLink)
@@ -2115,7 +2144,6 @@ create_callin_process (OOP contextOOP)
   /* Put initialProcessOOP in the root set */
   add_first_link (initialProcessListOOP, initialProcessOOP);
 
-  _gst_invalidate_method_cache ();
   return (initialProcessOOP);
 }
 
@@ -2138,6 +2166,8 @@ _gst_get_var (enum gst_var_index index)
       return (_gst_make_core_file);
     case GST_REGRESSION_TESTING:
       return (_gst_regression_testing);
+    case GST_NO_LINE_NUMBERS:
+      return (_gst_omit_line_numbers);
     default:
       return (-1);
     }
@@ -2173,6 +2203,9 @@ _gst_set_var (enum gst_var_index index, int value)
     case GST_REGRESSION_TESTING:
       _gst_regression_testing = true;
       break;
+    case GST_NO_LINE_NUMBERS:
+      _gst_omit_line_numbers = value;
+      break;
     default:
       return (-1);
     }
@@ -2194,12 +2227,6 @@ _gst_init_interpreter (void)
 #endif
 
   _gst_this_context_oop = _gst_nil_oop;
-  async_queue_index = 0;
-  async_queue_index_sig = 0;
-  async_queue_size = 32;
-  queued_async_signals = malloc (sizeof (async_queue_entry) * async_queue_size);
-  gl_lock_init (async_queue_lock);
-
   for (i = 0; i < MAX_LIFO_DEPTH; i++)
     lifo_contexts[i].flags = F_POOLED | F_CONTEXT;
 
@@ -2318,7 +2345,7 @@ set_preemption_timer (void)
 
   time_to_preempt = false;
   if (timeSlice > 0)
-    _gst_signal_after (timeSlice, preempt_smalltalk_process, SIGVTALRM);
+    _gst_sigvtalrm_every (timeSlice, preempt_smalltalk_process);
 #endif
 }
 
@@ -2368,16 +2395,14 @@ _gst_copy_processor_registers (void)
 void
 copy_semaphore_oops (void)
 {
-  int i;
+  async_queue_entry *sig;
 
-  _gst_disable_interrupts (false);	/* block out everything! */
-
-  for (i = 0; i < async_queue_index; i++)
-    if (queued_async_signals[i].data)
-      MAYBE_COPY_OOP (queued_async_signals[i].data);
-  for (i = 0; i < async_queue_index_sig; i++)
-    if (queued_async_signals_sig[i].data)
-      MAYBE_COPY_OOP (queued_async_signals_sig[i].data);
+  for (sig = queued_async_signals; sig != &queued_async_signals_tail;
+       sig = sig->next)
+    MAYBE_COPY_OOP (sig->data);
+  for (sig = queued_async_signals_sig; sig != &queued_async_signals_tail;
+       sig = sig->next)
+    MAYBE_COPY_OOP (sig->data);
 
   /* there does seem to be a window where this is not valid */
   if (single_step_semaphore)
@@ -2385,8 +2410,6 @@ copy_semaphore_oops (void)
 
   /* there does seem to be a window where this is not valid */
   MAYBE_COPY_OOP (switch_to_process);
-
-  _gst_enable_interrupts (false);
 }
 
 
@@ -2406,16 +2429,14 @@ _gst_mark_processor_registers (void)
 void
 mark_semaphore_oops (void)
 {
-  int i;
+  async_queue_entry *sig;
 
-  _gst_disable_interrupts (false);	/* block out everything! */
-
-  for (i = 0; i < async_queue_index; i++)
-    if (queued_async_signals[i].data)
-      MAYBE_MARK_OOP (queued_async_signals[i].data);
-  for (i = 0; i < async_queue_index_sig; i++)
-    if (queued_async_signals_sig[i].data)
-      MAYBE_MARK_OOP (queued_async_signals_sig[i].data);
+  for (sig = queued_async_signals; sig != &queued_async_signals_tail;
+       sig = sig->next)
+    MAYBE_MARK_OOP (sig->data);
+  for (sig = queued_async_signals_sig; sig != &queued_async_signals_tail;
+       sig = sig->next)
+    MAYBE_MARK_OOP (sig->data);
 
   /* there does seem to be a window where this is not valid */
   if (single_step_semaphore)
@@ -2423,8 +2444,6 @@ mark_semaphore_oops (void)
 
   /* there does seem to be a window where this is not valid */
   MAYBE_MARK_OOP (switch_to_process);
-
-  _gst_enable_interrupts (false);
 }
 
 
@@ -2779,13 +2798,31 @@ stop_execution (void)
 }
 
 mst_Boolean
-parse_stream_with_protection (mst_Boolean method)
+parse_method_from_stream_with_protection (OOP currentClass,
+					  OOP currentCategory)
+{
+  interp_jmp_buf jb;
+  OOP methodOOP;
+
+  push_jmp_buf (&jb, false, get_active_process ());
+  if (setjmp (jb.jmpBuf) == 0)
+    methodOOP = _gst_parse_method_from_stream (currentClass,
+                                               currentCategory);
+  else
+    methodOOP = _gst_nil_oop;
+
+  PUSH_OOP (methodOOP);
+  return pop_jmp_buf ();
+}
+
+mst_Boolean
+parse_stream_with_protection (OOP currentNamespace)
 {
   interp_jmp_buf jb;
 
   push_jmp_buf (&jb, false, get_active_process ());
   if (setjmp (jb.jmpBuf) == 0)
-    _gst_parse_stream (method);
+    _gst_parse_stream (currentNamespace);
 
   return pop_jmp_buf ();
 }
